"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AreaHighlight = void 0;
const react_1 = __importDefault(require("react"));
const pdfjs_dom_1 = require("../lib/pdfjs-dom");
require("../style/AreaHighlight.css");
const react_rnd_1 = require("react-rnd");
/**
 * Renders a resizeable and interactive rectangular area for a highlight.
 *
 * @category Component
 */
const AreaHighlight = ({ highlight, onChange, isScrolledTo, bounds, onContextMenu, onEditStart, style, }) => {
    const highlightClass = isScrolledTo ? "AreaHighlight--scrolledTo" : "";
    // Generate key based on position. This forces a remount (and a defaultpos update)
    // whenever highlight position changes (e.g., when updated, scale changes, etc.)
    // We don't use position as state because when updating Rnd this would happen and cause flickering:
    // User moves Rnd -> Rnd records new pos -> Rnd jumps back -> highlight updates -> Rnd re-renders at new pos
    const key = `${highlight.position.boundingRect.width}${highlight.position.boundingRect.height}${highlight.position.boundingRect.left}${highlight.position.boundingRect.top}`;
    return (react_1.default.createElement("div", { className: `AreaHighlight ${highlightClass}`, onContextMenu: onContextMenu },
        react_1.default.createElement(react_rnd_1.Rnd, { className: "AreaHighlight__part", onDragStop: (_, data) => {
                const boundingRect = {
                    ...highlight.position.boundingRect,
                    top: data.y,
                    left: data.x,
                };
                onChange && onChange(boundingRect);
            }, onResizeStop: (_mouseEvent, _direction, ref, _delta, position) => {
                const boundingRect = {
                    top: position.y,
                    left: position.x,
                    width: ref.offsetWidth,
                    height: ref.offsetHeight,
                    pageNumber: (0, pdfjs_dom_1.getPageFromElement)(ref)?.number || -1,
                };
                onChange && onChange(boundingRect);
            }, onDragStart: onEditStart, onResizeStart: onEditStart, default: {
                x: highlight.position.boundingRect.left,
                y: highlight.position.boundingRect.top,
                width: highlight.position.boundingRect.width,
                height: highlight.position.boundingRect.height,
            }, key: key, bounds: bounds, 
            // Prevevent any event clicks as clicking is already used for movement
            onClick: (event) => {
                event.stopPropagation();
                event.preventDefault();
            }, style: style })));
};
exports.AreaHighlight = AreaHighlight;
//# sourceMappingURL=AreaHighlight.js.map