"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HighlightLayer = void 0;
const react_1 = __importDefault(require("react"));
const HighlightContext_1 = require("../contexts/HighlightContext");
const coordinates_1 = require("../lib/coordinates");
const screenshot_1 = __importDefault(require("../lib/screenshot"));
const EMPTY_ID = "empty-id";
/**
 * A component responsible for managing all the highlights and ghost highlights
 * for a single page of a PDF document. It does not render each highlight
 * but it provides context for a highlight container to do so.
 * Its rendering should be controlled by a {@link PdfHighlighter}.
 *
 * @category Component
 * @internal
 */
const HighlightLayer = ({ highlightsByPage, pageNumber, scrolledToHighlightId, viewer, highlightBindings, children, }) => {
    const currentHighlights = highlightsByPage[pageNumber] || [];
    return (react_1.default.createElement("div", null, currentHighlights.map((highlight, index) => {
        const viewportHighlight = {
            ...highlight,
            id: "id" in highlight ? highlight.id : EMPTY_ID, // Give Empty ID to GhostHighlight
            position: (0, coordinates_1.scaledPositionToViewport)(highlight.position, viewer),
        };
        const isScrolledTo = Boolean(scrolledToHighlightId === viewportHighlight.id);
        const highlightUtils = {
            highlight: viewportHighlight,
            viewportToScaled: (rect) => {
                const viewport = viewer.getPageView((rect.pageNumber || pageNumber) - 1).viewport;
                return (0, coordinates_1.viewportToScaled)(rect, viewport);
            },
            screenshot: (boundingRect) => (0, screenshot_1.default)(boundingRect, pageNumber, viewer),
            isScrolledTo: isScrolledTo,
            highlightBindings,
        };
        return (react_1.default.createElement(HighlightContext_1.HighlightContext.Provider, { value: highlightUtils, key: index }, children));
    })));
};
exports.HighlightLayer = HighlightLayer;
//# sourceMappingURL=HighlightLayer.js.map