"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MonitoredHighlightContainer = void 0;
const react_1 = __importStar(require("react"));
const PdfHighlighterContext_1 = require("../contexts/PdfHighlighterContext");
const MouseMonitor_1 = require("./MouseMonitor");
/**
 * A container for a highlight component that monitors whether a mouse is over a
 * highlight and over some secondary highlight tip. It will display the tip
 * whenever the mouse is over the highlight and it will hide the tip only when
 * the mouse has left the highlight AND the tip.
 *
 * @category Component
 */
const MonitoredHighlightContainer = ({ onMouseEnter, highlightTip, onMouseLeave, children, }) => {
    const mouseInRef = (0, react_1.useRef)(false); // Whether the mouse is over the child (highlight)
    const { setTip, isEditingOrHighlighting } = (0, PdfHighlighterContext_1.usePdfHighlighterContext)();
    return (react_1.default.createElement("div", { onMouseEnter: () => {
            mouseInRef.current = true;
            onMouseEnter && onMouseEnter();
            if (isEditingOrHighlighting())
                return;
            if (highlightTip) {
                // MouseMonitor the highlightTip to prevent it from disappearing if the mouse is over it and not the highlight.
                const monitoredHighlightTip = (react_1.default.createElement(MouseMonitor_1.MouseMonitor, { onMoveAway: () => {
                        // The event will keep triggering if the mouse is not on the highlightTip,
                        // but don't do anything if the mouse is over the highlight.
                        if (mouseInRef.current) {
                            return;
                        }
                        setTip(null);
                        onMouseLeave && onMouseLeave();
                    }, paddingX: 60, paddingY: 30 }, highlightTip.content));
                setTip({
                    position: highlightTip.position,
                    content: monitoredHighlightTip,
                });
            }
        }, onMouseLeave: () => {
            mouseInRef.current = false;
            // Trigger onMouseLeave if no highlightTip exists
            !highlightTip && onMouseLeave && onMouseLeave();
        } }, children));
};
exports.MonitoredHighlightContainer = MonitoredHighlightContainer;
//# sourceMappingURL=MonitoredHighlightContainer.js.map