"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MouseSelection = void 0;
const react_1 = __importStar(require("react"));
const pdfjs_dom_1 = require("../lib/pdfjs-dom");
require("../style/MouseSelection.css");
const coordinates_1 = require("../lib/coordinates");
const screenshot_1 = __importDefault(require("../lib/screenshot"));
const getBoundingRect = (start, end) => {
    return {
        left: Math.min(end.x, start.x),
        top: Math.min(end.y, start.y),
        width: Math.abs(end.x - start.x),
        height: Math.abs(end.y - start.y),
    };
};
const getContainerCoords = (container, pageX, pageY) => {
    const containerBoundingRect = container.getBoundingClientRect();
    return {
        x: pageX - containerBoundingRect.left + container.scrollLeft,
        y: pageY - containerBoundingRect.top + container.scrollTop - window.scrollY,
    };
};
/**
 * A component that enables the creation of rectangular and interactive mouse
 * selections within a given container. NOTE: This does not disable selection in
 * whatever container the component is placed in. That must be handled through
 * the component's events.
 *
 * @category Component
 * @internal
 */
const MouseSelection = ({ viewer, onSelection, onReset, onDragStart, enableAreaSelection, onChange, style, }) => {
    const [start, setStart] = (0, react_1.useState)(null);
    const [end, setEnd] = (0, react_1.useState)(null);
    const [locked, setLocked] = (0, react_1.useState)(false);
    const rootRef = (0, react_1.useRef)(null);
    // Needed in order to grab the page info of a mouse selection
    const startTargetRef = (0, react_1.useRef)(null);
    const reset = () => {
        onReset && onReset();
        setStart(null);
        setEnd(null);
        setLocked(false);
    };
    // Register event listeners onChange
    (0, react_1.useEffect)(() => {
        onChange && onChange(Boolean(start && end));
        if (!rootRef.current)
            return;
        // Should be the PdfHighlighter
        const container = (0, pdfjs_dom_1.asElement)(rootRef.current.parentElement);
        const handleMouseUp = (event) => {
            if (!start || !end || !startTargetRef.current)
                return;
            const boundingRect = getBoundingRect(start, end);
            // Check if the bounding rectangle has a minimum width and height
            // to prevent recording selections with 0 area
            const shouldEnd = boundingRect.width >= 1 && boundingRect.height >= 1;
            if (!container.contains((0, pdfjs_dom_1.asElement)(event.target)) || !shouldEnd) {
                reset();
                return;
            }
            setLocked(true);
            const page = (0, pdfjs_dom_1.getPageFromElement)(startTargetRef.current);
            if (!page)
                return;
            const pageBoundingRect = {
                ...boundingRect,
                top: boundingRect.top - page.node.offsetTop,
                left: boundingRect.left - page.node.offsetLeft,
                pageNumber: page.number,
            };
            const viewportPosition = {
                boundingRect: pageBoundingRect,
                rects: [],
            };
            const scaledPosition = (0, coordinates_1.viewportPositionToScaled)(viewportPosition, viewer);
            const image = (0, screenshot_1.default)(pageBoundingRect, pageBoundingRect.pageNumber, viewer);
            onSelection &&
                onSelection(viewportPosition, scaledPosition, image, reset, event);
        };
        const handleMouseMove = (event) => {
            if (!rootRef.current || !start || locked)
                return;
            setEnd(getContainerCoords(container, event.pageX, event.pageY));
        };
        const handleMouseDown = (event) => {
            const shouldStart = (event) => enableAreaSelection(event) &&
                (0, pdfjs_dom_1.isHTMLElement)(event.target) &&
                Boolean((0, pdfjs_dom_1.asElement)(event.target).closest(".page"));
            // If the user clicks anywhere outside a tip, reset the selection
            const shouldReset = (event) => start &&
                !(0, pdfjs_dom_1.asElement)(event.target).closest(".PdfHighlighter__tip-container");
            if (!shouldStart(event)) {
                if (shouldReset(event))
                    reset();
                return;
            }
            startTargetRef.current = (0, pdfjs_dom_1.asElement)(event.target);
            onDragStart && onDragStart(event);
            setStart(getContainerCoords(container, event.pageX, event.pageY));
            setEnd(null);
            setLocked(false);
        };
        /**
         * Although we register the event listeners on the PdfHighlighter component, we encapsulate
         * them in this separate component to enhance maintainability and prevent unnecessary
         * rerenders of the PdfHighlighter itself. While synthetic events on PdfHighlighter would
         * be preferable, we need to register "mouseup" on the entire document anyway. Therefore,
         * we can't avoid using useEffect. We must re-register all events on state changes, as
         * custom event listeners may otherwise receive stale state.
         */
        container.addEventListener("mousemove", handleMouseMove);
        container.addEventListener("mousedown", handleMouseDown);
        document.addEventListener("mouseup", handleMouseUp);
        return () => {
            container.removeEventListener("mousemove", handleMouseMove);
            container.removeEventListener("mousedown", handleMouseDown);
            document.removeEventListener("mouseup", handleMouseUp);
        };
    }, [start, end]);
    return (react_1.default.createElement("div", { className: "MouseSelection-container", ref: rootRef }, start && end && (react_1.default.createElement("div", { className: "MouseSelection", style: { ...getBoundingRect(start, end), ...style } }))));
};
exports.MouseSelection = MouseSelection;
//# sourceMappingURL=MouseSelection.js.map