"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PdfHighlighter = void 0;
require("pdfjs-dist/web/pdf_viewer.css");
require("../style/PdfHighlighter.css");
require("../style/pdf_viewer.css");
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const pdf_viewer_1 = require("pdfjs-dist/legacy/web/pdf_viewer");
const react_1 = __importStar(require("react"));
const client_1 = require("react-dom/client");
const PdfHighlighterContext_1 = require("../contexts/PdfHighlighterContext");
const coordinates_1 = require("../lib/coordinates");
const disable_text_selection_1 = require("../lib/disable-text-selection");
const get_bounding_rect_1 = __importDefault(require("../lib/get-bounding-rect"));
const get_client_rects_1 = __importDefault(require("../lib/get-client-rects"));
const group_highlights_by_page_1 = __importDefault(require("../lib/group-highlights-by-page"));
const pdfjs_dom_1 = require("../lib/pdfjs-dom");
const HighlightLayer_1 = require("./HighlightLayer");
const MouseSelection_1 = require("./MouseSelection");
const TipContainer_1 = require("./TipContainer");
const SCROLL_MARGIN = 10;
const SELECTION_DELAY = 250; // Debounce wait time in milliseconds for a selection changing to be registered
const DEFAULT_SCALE_VALUE = "auto";
const DEFAULT_TEXT_SELECTION_COLOR = "rgba(153,193,218,255)";
const findOrCreateHighlightLayer = (textLayer) => {
    return (0, pdfjs_dom_1.findOrCreateContainerLayer)(textLayer, "PdfHighlighter__highlight-layer");
};
/**
 * This is a large-scale PDF viewer component designed to facilitate
 * highlighting. It should be used as a child to a {@link PdfLoader} to ensure
 * proper document loading. This does not itself render any highlights, but
 * instead its child should be the container component for each individual
 * highlight. This component will be provided appropriate HighlightContext for
 * rendering.
 *
 * @category Component
 */
const PdfHighlighter = ({ highlights, onScrollAway, pdfScaleValue = DEFAULT_SCALE_VALUE, onSelection: onSelectionFinished, onCreateGhostHighlight, onRemoveGhostHighlight, selectionTip, enableAreaSelection, mouseSelectionStyle, pdfDocument, children, textSelectionColor = DEFAULT_TEXT_SELECTION_COLOR, utilsRef, style, }) => {
    // State
    const [tip, setTip] = (0, react_1.useState)(null);
    const [isViewerReady, setIsViewerReady] = (0, react_1.useState)(false);
    // Refs
    const containerNodeRef = (0, react_1.useRef)(null);
    const highlightBindingsRef = (0, react_1.useRef)({});
    const ghostHighlightRef = (0, react_1.useRef)(null);
    const selectionRef = (0, react_1.useRef)(null);
    const scrolledToHighlightIdRef = (0, react_1.useRef)(null);
    const isAreaSelectionInProgressRef = (0, react_1.useRef)(false);
    const isEditInProgressRef = (0, react_1.useRef)(false);
    const updateTipPositionRef = (0, react_1.useRef)(() => { });
    const eventBusRef = (0, react_1.useRef)(new pdf_viewer_1.EventBus());
    const linkServiceRef = (0, react_1.useRef)(new pdf_viewer_1.PDFLinkService({
        eventBus: eventBusRef.current,
        externalLinkTarget: 2,
    }));
    const resizeObserverRef = (0, react_1.useRef)(null);
    const viewerRef = (0, react_1.useRef)(null);
    // Initialise PDF Viewer
    (0, react_1.useLayoutEffect)(() => {
        if (!containerNodeRef.current)
            return;
        const debouncedDocumentInit = (0, lodash_debounce_1.default)(() => {
            viewerRef.current =
                viewerRef.current ||
                    new pdf_viewer_1.PDFViewer({
                        container: containerNodeRef.current,
                        eventBus: eventBusRef.current,
                        // @ts-ignore enhanceTextSelection is deprecated but this is the last version to support it. See: https://github.com/DanielArnould/react-pdf-highlighter-extended/issues/3
                        enhanceTextSelection: true,
                        textLayerMode: 2,
                        removePageBorders: true,
                        linkService: linkServiceRef.current,
                        l10n: pdf_viewer_1.NullL10n, // No localisation
                    });
            viewerRef.current.setDocument(pdfDocument);
            linkServiceRef.current.setDocument(pdfDocument);
            linkServiceRef.current.setViewer(viewerRef.current);
            setIsViewerReady(true);
        }, 100);
        debouncedDocumentInit();
        return () => {
            debouncedDocumentInit.cancel();
        };
    }, [document]);
    // Initialise viewer event listeners
    (0, react_1.useLayoutEffect)(() => {
        if (!containerNodeRef.current)
            return;
        resizeObserverRef.current = new ResizeObserver(handleScaleValue);
        resizeObserverRef.current.observe(containerNodeRef.current);
        const doc = containerNodeRef.current.ownerDocument;
        eventBusRef.current.on("textlayerrendered", renderHighlightLayers);
        eventBusRef.current.on("pagesinit", handleScaleValue);
        doc.addEventListener("selectionchange", debouncedHandleSelectionChange);
        doc.addEventListener("keydown", handleKeyDown);
        renderHighlightLayers();
        return () => {
            eventBusRef.current.off("pagesinit", handleScaleValue);
            eventBusRef.current.off("textlayerrendered", renderHighlightLayers);
            doc.removeEventListener("selectionchange", debouncedHandleSelectionChange);
            doc.removeEventListener("keydown", handleKeyDown);
            resizeObserverRef.current?.disconnect();
        };
    }, [selectionTip, highlights, onSelectionFinished]);
    // Event listeners
    const handleScroll = () => {
        onScrollAway && onScrollAway();
        scrolledToHighlightIdRef.current = null;
        renderHighlightLayers();
    };
    const debouncedHandleSelectionChange = (0, lodash_debounce_1.default)(() => {
        const container = containerNodeRef.current;
        const selection = (0, pdfjs_dom_1.getWindow)(container).getSelection();
        if (!container || !selection || selection.isCollapsed || !viewerRef.current)
            return;
        const range = selection.rangeCount > 0 ? selection.getRangeAt(0) : null;
        // Check the selected text is in the document, not the tip
        if (!range || !container.contains(range.commonAncestorContainer))
            return;
        const pages = (0, pdfjs_dom_1.getPagesFromRange)(range);
        if (!pages || pages.length === 0)
            return;
        const rects = (0, get_client_rects_1.default)(range, pages);
        if (rects.length === 0)
            return;
        const viewportPosition = {
            boundingRect: (0, get_bounding_rect_1.default)(rects),
            rects,
        };
        const scaledPosition = (0, coordinates_1.viewportPositionToScaled)(viewportPosition, viewerRef.current);
        const content = {
            text: selection.toString().split("\n").join(" "), // Make all line breaks spaces
        };
        selectionRef.current = {
            content,
            position: scaledPosition,
            makeGhostHighlight: () => {
                ghostHighlightRef.current = {
                    content: content,
                    position: scaledPosition,
                };
                onCreateGhostHighlight &&
                    onCreateGhostHighlight(ghostHighlightRef.current);
                clearTextSelection();
                renderHighlightLayers();
                return ghostHighlightRef.current;
            },
        };
        onSelectionFinished && onSelectionFinished(selectionRef.current);
        selectionTip &&
            setTip({ position: viewportPosition, content: selectionTip });
    }, SELECTION_DELAY);
    const handleMouseDown = (event) => {
        if (!(0, pdfjs_dom_1.isHTMLElement)(event.target) ||
            (0, pdfjs_dom_1.asElement)(event.target).closest(".PdfHighlighter__tip-container") // Ignore selections on tip container
        ) {
            return;
        }
        setTip(null);
        clearTextSelection(); // TODO: Check if clearing text selection only if not clicking on tip breaks anything.
        removeGhostHighlight();
        toggleEditInProgress(false);
    };
    const handleKeyDown = (event) => {
        if (event.code === "Escape") {
            clearTextSelection();
            removeGhostHighlight();
            setTip(null);
        }
    };
    const handleScaleValue = () => {
        if (viewerRef.current) {
            viewerRef.current.currentScaleValue = pdfScaleValue.toString();
        }
    };
    // Render Highlight layers
    const renderHighlightLayer = (highlightBindings, pageNumber) => {
        if (!viewerRef.current)
            return;
        highlightBindings.reactRoot.render(react_1.default.createElement(PdfHighlighterContext_1.PdfHighlighterContext.Provider, { value: pdfHighlighterUtils },
            react_1.default.createElement(HighlightLayer_1.HighlightLayer, { highlightsByPage: (0, group_highlights_by_page_1.default)([
                    ...highlights,
                    ghostHighlightRef.current,
                ]), pageNumber: pageNumber, scrolledToHighlightId: scrolledToHighlightIdRef.current, viewer: viewerRef.current, highlightBindings: highlightBindings, children: children })));
    };
    const renderHighlightLayers = () => {
        if (!viewerRef.current)
            return;
        for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
            const highlightBindings = highlightBindingsRef.current[pageNumber];
            // Need to check if container is still attached to the DOM as PDF.js can unload pages.
            if (highlightBindings?.container?.isConnected) {
                renderHighlightLayer(highlightBindings, pageNumber);
            }
            else {
                const { textLayer } = viewerRef.current.getPageView(pageNumber - 1) || {};
                if (!textLayer)
                    continue; // Viewer hasn't rendered page yet
                // textLayer.div for version >=3.0 and textLayer.textLayerDiv otherwise.
                const highlightLayer = findOrCreateHighlightLayer(textLayer.textLayerDiv);
                if (highlightLayer) {
                    const reactRoot = (0, client_1.createRoot)(highlightLayer);
                    highlightBindingsRef.current[pageNumber] = {
                        reactRoot,
                        container: highlightLayer,
                        textLayer: textLayer.textLayerDiv, // textLayer.div for version >=3.0 and textLayer.textLayerDiv otherwise.
                    };
                    renderHighlightLayer(highlightBindingsRef.current[pageNumber], pageNumber);
                }
            }
        }
    };
    // Utils
    const isEditingOrHighlighting = () => {
        return (Boolean(selectionRef.current) ||
            Boolean(ghostHighlightRef.current) ||
            isAreaSelectionInProgressRef.current ||
            isEditInProgressRef.current);
    };
    const toggleEditInProgress = (flag) => {
        if (flag !== undefined) {
            isEditInProgressRef.current = flag;
        }
        else {
            isEditInProgressRef.current = !isEditInProgressRef.current;
        }
        // Disable text selection
        if (viewerRef.current)
            viewerRef.current.viewer?.classList.toggle("PdfHighlighter--disable-selection", isEditInProgressRef.current);
    };
    const removeGhostHighlight = () => {
        if (onRemoveGhostHighlight && ghostHighlightRef.current)
            onRemoveGhostHighlight(ghostHighlightRef.current);
        ghostHighlightRef.current = null;
        renderHighlightLayers();
    };
    const clearTextSelection = () => {
        selectionRef.current = null;
        const container = containerNodeRef.current;
        const selection = (0, pdfjs_dom_1.getWindow)(container).getSelection();
        if (!container || !selection)
            return;
        selection.removeAllRanges();
    };
    const scrollToHighlight = (highlight) => {
        const { boundingRect, usePdfCoordinates } = highlight.position;
        const pageNumber = boundingRect.pageNumber;
        // Remove scroll listener in case user auto-scrolls in succession.
        viewerRef.current.container.removeEventListener("scroll", handleScroll);
        const pageViewport = viewerRef.current.getPageView(pageNumber - 1).viewport;
        console.log(pageViewport);
        viewerRef.current.scrollPageIntoView({
            pageNumber,
            destArray: [
                null, // null since we pass pageNumber already as an arg
                { name: "XYZ" },
                ...pageViewport.convertToPdfPoint(0, // Default x coord
                (0, coordinates_1.scaledToViewport)(boundingRect, pageViewport, usePdfCoordinates).top -
                    SCROLL_MARGIN),
                0, // Default z coord
            ],
        });
        scrolledToHighlightIdRef.current = highlight.id;
        renderHighlightLayers();
        // wait for scrolling to finish
        setTimeout(() => {
            viewerRef.current.container.addEventListener("scroll", handleScroll, {
                once: true,
            });
        }, 100);
    };
    const pdfHighlighterUtils = {
        isEditingOrHighlighting,
        getCurrentSelection: () => selectionRef.current,
        getGhostHighlight: () => ghostHighlightRef.current,
        removeGhostHighlight,
        toggleEditInProgress,
        isEditInProgress: () => isEditInProgressRef.current,
        isSelectionInProgress: () => Boolean(selectionRef.current) || isAreaSelectionInProgressRef.current,
        scrollToHighlight,
        getViewer: () => viewerRef.current,
        getTip: () => tip,
        setTip,
        updateTipPosition: updateTipPositionRef.current,
    };
    utilsRef(pdfHighlighterUtils);
    return (react_1.default.createElement(PdfHighlighterContext_1.PdfHighlighterContext.Provider, { value: pdfHighlighterUtils },
        react_1.default.createElement("div", { ref: containerNodeRef, className: "PdfHighlighter", onPointerDown: handleMouseDown, style: style },
            react_1.default.createElement("div", { className: "pdfViewer" }),
            react_1.default.createElement("style", null, `
          .textLayer ::selection {
            background: ${textSelectionColor};
          }
        `),
            isViewerReady && (react_1.default.createElement(TipContainer_1.TipContainer, { viewer: viewerRef.current, updateTipPositionRef: updateTipPositionRef })),
            isViewerReady && enableAreaSelection && (react_1.default.createElement(MouseSelection_1.MouseSelection, { viewer: viewerRef.current, onChange: (isVisible) => (isAreaSelectionInProgressRef.current = isVisible), enableAreaSelection: enableAreaSelection, style: mouseSelectionStyle, onDragStart: () => (0, disable_text_selection_1.disableTextSelection)(viewerRef.current, true), onReset: () => {
                    selectionRef.current = null;
                    (0, disable_text_selection_1.disableTextSelection)(viewerRef.current, false);
                }, onSelection: (viewportPosition, scaledPosition, image, resetSelection) => {
                    selectionRef.current = {
                        content: { image },
                        position: scaledPosition,
                        makeGhostHighlight: () => {
                            ghostHighlightRef.current = {
                                position: scaledPosition,
                                content: { image },
                            };
                            onCreateGhostHighlight &&
                                onCreateGhostHighlight(ghostHighlightRef.current);
                            resetSelection();
                            renderHighlightLayers();
                            return ghostHighlightRef.current;
                        },
                    };
                    onSelectionFinished && onSelectionFinished(selectionRef.current);
                    selectionTip &&
                        setTip({ position: viewportPosition, content: selectionTip });
                } })))));
};
exports.PdfHighlighter = PdfHighlighter;
//# sourceMappingURL=PdfHighlighter.js.map