"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PdfLoader = void 0;
const react_1 = __importStar(require("react"));
const pdf_1 = require("pdfjs-dist/legacy/build/pdf");
const DEFAULT_BEFORE_LOAD = (progress) => (react_1.default.createElement("div", { style: { color: "black" } },
    "Loading ",
    Math.floor((progress.loaded / progress.total) * 100),
    "%"));
const DEFAULT_ERROR_MESSAGE = (error) => (react_1.default.createElement("div", { style: { color: "black" } }, error.message));
const DEFAULT_ON_ERROR = (error) => {
    throw new Error(`Error loading PDF document: ${error.message}!`);
};
const DEFAULT_WORKER_SRC = "https://unpkg.com/pdfjs-dist@2.16.105/build/pdf.worker.min.js";
/**
 * A component for loading a PDF document and passing it to a child.
 *
 * @category Component
 */
const PdfLoader = ({ document, beforeLoad = DEFAULT_BEFORE_LOAD, errorMessage = DEFAULT_ERROR_MESSAGE, children, onError = DEFAULT_ON_ERROR, workerSrc = DEFAULT_WORKER_SRC, }) => {
    const pdfLoadingTaskRef = (0, react_1.useRef)(null);
    const pdfDocumentRef = (0, react_1.useRef)(null);
    const [error, setError] = (0, react_1.useState)(null);
    const [loadingProgress, setLoadingProgress] = (0, react_1.useState)(null);
    // Intitialise document
    (0, react_1.useEffect)(() => {
        pdf_1.GlobalWorkerOptions.workerSrc = workerSrc;
        pdfLoadingTaskRef.current = (0, pdf_1.getDocument)(document);
        pdfLoadingTaskRef.current.onProgress = (progress) => {
            setLoadingProgress(progress.loaded > progress.total ? null : progress);
        };
        pdfLoadingTaskRef.current.promise
            .then((pdfDocument) => {
            pdfDocumentRef.current = pdfDocument;
        })
            .catch((error) => {
            if (error.message != "Worker was destroyed") {
                setError(error);
                onError(error);
            }
        })
            .finally(() => {
            setLoadingProgress(null);
        });
        return () => {
            if (pdfLoadingTaskRef.current) {
                pdfLoadingTaskRef.current.destroy();
            }
            if (pdfDocumentRef.current) {
                pdfDocumentRef.current.destroy();
            }
        };
    }, [document]);
    return error
        ? errorMessage(error)
        : loadingProgress
            ? beforeLoad(loadingProgress)
            : pdfDocumentRef.current && children(pdfDocumentRef.current);
};
exports.PdfLoader = PdfLoader;
//# sourceMappingURL=PdfLoader.js.map