"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TipContainer = void 0;
const react_1 = __importStar(require("react"));
const PdfHighlighterContext_1 = require("../contexts/PdfHighlighterContext");
const clamp = (value, left, right) => Math.min(Math.max(value, left), right);
const VERTICAL_PADDING = 5;
/**
 * A component that manages rendering and placement of a tip around a highlight.
 * It does not automatically update the tip's position if it resizes.
 *
 * @category Component
 * @internal
 */
const TipContainer = ({ viewer, updateTipPositionRef, }) => {
    const [height, setHeight] = (0, react_1.useState)(0);
    const [width, setWidth] = (0, react_1.useState)(0);
    const containerRef = (0, react_1.useRef)(null);
    const updatePosition = () => {
        if (!containerRef.current)
            return;
        const { offsetHeight, offsetWidth } = containerRef.current;
        setHeight(offsetHeight);
        setWidth(offsetWidth);
    };
    updateTipPositionRef.current = updatePosition;
    // useLayoutEffect ensures state is updated before re-render, preventing flickering
    (0, react_1.useLayoutEffect)(() => {
        updatePosition();
    }, [updatePosition]);
    const { getTip } = (0, PdfHighlighterContext_1.usePdfHighlighterContext)();
    const currentTip = getTip();
    if (!currentTip)
        return null;
    // Destructuring current tip's position and content
    const { position, content } = currentTip;
    const { boundingRect } = position;
    const pageNumber = boundingRect.pageNumber;
    const pageNode = viewer.getPageView(pageNumber - 1).div; // Account for 1 indexing of pdf documents
    const pageBoundingClientRect = pageNode.getBoundingClientRect();
    const { left: pageLeft, width: pageWidth } = pageBoundingClientRect;
    // Calculate the position and dimensions of the tip container
    const scrollTop = viewer.container.scrollTop; // How much the viewer has been scrolled vertically
    const left = pageNode.offsetLeft + boundingRect.left + boundingRect.width / 2; // center tip over highlight
    const highlightTop = boundingRect.top + pageNode.offsetTop;
    const highlightBottom = highlightTop + boundingRect.height;
    // Determine whether the tip should be moved below the highlight
    const shouldMove = highlightTop - height - VERTICAL_PADDING < scrollTop; // Would the tip render beyond the top of the visible document?
    const top = shouldMove
        ? highlightBottom + VERTICAL_PADDING
        : highlightTop - height - VERTICAL_PADDING;
    // Ensure the tip stays within the left edge of the viewer and the right edge of the page
    const clampedLeft = clamp(left - width / 2, 0, pageLeft + pageWidth - width);
    return (react_1.default.createElement("div", { className: "PdfHighlighter__tip-container", style: {
            top,
            left: clampedLeft,
            height: "max-content",
            width: "max-content",
        }, ref: containerRef }, content));
};
exports.TipContainer = TipContainer;
//# sourceMappingURL=TipContainer.js.map